/* lsDemo1.cpp - Demonstration RTX Extension indicator created by Chad Payne. Created: 07/14/2015 12:24:57 */

#include "irtsdk.h"
#include <math.h>

#define max(a,b)            (((a) > (b)) ? (a) : (b))
#define min(a,b)            (((a) < (b)) ? (a) : (b))

class myRTExtension : public cppExtension {
public:
	myRTExtension() : cppExtension() {}
	virtual int	parmsUpdt(unsigned int iParmNumber);
	virtual int	parmsLoad(void);
	virtual int parmsInit(unsigned int iVersion);
	virtual int keyboard(RTX_EVENT *p);
	virtual int mouse(RTX_EVENT *p);
	virtual int	draw(void);
};

/* enumeration below should match the persistent parameters in the structure below... */
/* the elements of this enum are used in parmsLoad and parmsUpdt functions to identify controls when moving into position, hiding/showing, or enable/disabling */
typedef enum {	
	DEMO_PARM_SPERIOD,
	DEMO_PARM_BDRAWLINE,
	DEMO_PARM_BDRAWDOT,
	DEMO_PARM_COLORLINE,
	DEMO_PARM_COLORDOT,
	DEMO_PARM_DOTWIDTH,
	DEMO_PARM_UNKN = SHRT_MAX	
} DEMO_PARM_CONTROL;

typedef struct _userdata {
	 short	sPeriod;
	 BOOL	bDrawLine;
	 BOOL	bDrawDot;
	 COLOR	colorLine;
	 COLOR	colorDot;
	 short	dotWidth;
	 /**** persistent parameters above - these should be reflected in kParmTokens below - S for Short (Period), B for BOOL (Checkbox), U for Unsigned Long (Color), etc. */
	 /**** runtime parameters below if applicable - do not include these in kParmTokens */
	 /**** runtime parameters are often used to pass data between functions - like values computed in calc function to the draw function - or positions computed in the draw function to the mouse function */
	 short	sRuntime;
} UserData, *UserDataPtr;

#define kParmTokens   "SBBUUS"

/*****************************/
int cppExtension::setup(void)
/*****************************/
{
/* call setup function for each control in the same order they appear in kParmTokens and PARM_CONTROL enumeration above */
setPeriodParameter("Period", 13, "1;2;3;4;5;6;7;8;9;10;11;12;13;15;18;20;23;25;30;35;40;50;100", 50);
setBoolParameter("Draw Line", true);
setBoolParameter("Draw Dot", true);
setColorParameter("Color", 0x000080ff, 0, kParmAppendSameLine); 
setColorParameter("Color", 0x00c6e2ff, 0, kParmAppendSameLine); 
setPeriodParameter("Width", 4, "1;2;3;4;5;6;7;8;9;10;11;12;13;15;18;20;23;25;30;35;40;50;100", 50);

/* Move the controls into position */
this->parmsLoad();

return RTX_OK;
}

/*****************************/
extern "C" cppExtension *CreateExtension(void)
/*****************************/
{
cppExtension *pMyRTExtension = new myRTExtension();
pMyRTExtension->setArrayCount(1);
pMyRTExtension->setFlags(NO_UI + POST_DRAWING + OVERLAY);
pMyRTExtension->setDescription("lsDemo - Demonstration RTX Extension Indicator by Chad Payne.");
pMyRTExtension->setVersion("1.0");
return pMyRTExtension;
}
/*****************************/
int cppExtension::init(void)
/*****************************/
{
setParameterVersion(10);

return RTX_OK;
}


/*****************************/
int myRTExtension::parmsInit(unsigned int iVersion)
/*****************************/
{
UserDataPtr		pUserData = (UserDataPtr) allocUserData(sizeof(UserData));
size_t			specSize = (size_t) &pUserData->sRuntime - (size_t) &pUserData->sPeriod;

getParameters(pUserData, specSize, kParmTokens, NULL);
if(iVersion < 11)
	{	
	/* Update parameters here when releasing a new version beyond 1.0 */
	saveParameters(pUserData, specSize, kParmTokens);
	}

return RTX_OK;
}

/*****************************/
int myRTExtension::parmsLoad(void)
/*****************************/
{
// call all parmsUpdt needed here
int		x, y, yStep = 22, xBase = 25, yTopControl = 75;
	
/* code below places controls in proper location */
x = xBase + 135;
y = yTopControl + yStep;
setParameterPositionCtl(DEMO_PARM_SPERIOD, x, y); 
x = xBase + 80;
y += yStep;
setParameterPositionCtl(DEMO_PARM_BDRAWLINE, x, y);
x += 115;
y += 4;
setParameterPositionCtl(DEMO_PARM_COLORLINE, x, y);
x = xBase + 80;
y += yStep;
setParameterPositionCtl(DEMO_PARM_BDRAWDOT, x, y);
x += 115;
y += 4;
setParameterPositionCtl(DEMO_PARM_COLORDOT, x, y);
x = xBase + 135;
y += yStep;
setParameterPositionCtl(DEMO_PARM_DOTWIDTH, x, y);

/* call all parmsUpdt needed here to properly initialize the state of all controls when opening the parameters / preferences window */
if (getReason() == REASON_PARMS_LOAD)
	{
	parmsUpdt(DEMO_PARM_BDRAWLINE);
	parmsUpdt(DEMO_PARM_BDRAWDOT);
	return RTX_OK;
	}

return RTX_OK;
}
/*****************************/
int myRTExtension::parmsUpdt(unsigned int iParmNumber)
/*****************************/
{
// unsigned int		index, parm;
RTX_CONTEXT			ctx = getContext();

/* Show/Hide and Enable/Disable controls here */
switch(iParmNumber) {
	case DEMO_PARM_BDRAWLINE:
		{
		BOOL isChecked = isBoxChecked(DEMO_PARM_BDRAWLINE);
		enableParameter(DEMO_PARM_COLORLINE, isChecked);
		}
		break;
	case DEMO_PARM_BDRAWDOT:
		{
		BOOL isChecked = isBoxChecked(DEMO_PARM_BDRAWDOT);
		enableParameter(DEMO_PARM_COLORDOT, isChecked);
		enableParameter(DEMO_PARM_DOTWIDTH, isChecked);
		}
		break;
	}

return RTX_OK;
}

/*****************************/
int myRTExtension::keyboard(RTX_EVENT *pEvent)
/*****************************/
{
UserDataPtr		pUserData = (UserDataPtr) allocUserData(sizeof(UserData));
size_t			specSize = (size_t) &pUserData->sRuntime - (size_t) &pUserData->sPeriod;
RCT				gswRect;

gswRect.getPaneRect();
switch (pEvent->v.chr.ch) {
	case K_UP:
		/* up arrow keyboard event or mouse wheel up event - increase the period by 1.  If shift key is down, increase the dot width by 1 */
		getParameters(pUserData, specSize, kParmTokens, NULL);
		if(pEvent->v.chr.shift)
			{
			pUserData->dotWidth++;
			saveParameters(pUserData, specSize, kParmTokens);
			gswRect.invalidateRect();
			}
		else
			{
			pUserData->sPeriod++;
			saveParameters(pUserData, specSize, kParmTokens);
			this->calc(0);
			gswRect.invalidateRect();
			}
		break;
	case K_DOWN:
		/* down arrow keyboard event or mouse wheel down event - decrease the period by 1.  If shift key is down, decrease the dot width by 1 */
		getParameters(pUserData, specSize, kParmTokens, NULL);
		if(pEvent->v.chr.shift)
			{
			if(pUserData->dotWidth >= 3)
				{
				pUserData->dotWidth--;
				saveParameters(pUserData, specSize, kParmTokens);
				gswRect.invalidateRect();
				}
			}
		else
			{
			if(pUserData->sPeriod >= 2)
				{
				pUserData->sPeriod--;
				saveParameters(pUserData, specSize, kParmTokens);
				this->calc(0);
				gswRect.invalidateRect();
				}
			}
		break;
	case K_LEFT:
	case K_RIGHT:
		break;
	}
return RTX_OK;
}
/*****************************/
int myRTExtension::mouse(RTX_EVENT *pEvent)
/*****************************/
{
PNT				mouse, pnt;  
UserDataPtr		pUserData = (UserDataPtr) allocUserData(sizeof(UserData));
size_t			specSize = (size_t) &pUserData->sRuntime - (size_t) &pUserData->sPeriod;

getParameters(pUserData, specSize, kParmTokens, NULL);

switch (pEvent->type) {
	case E_MOUSE_MOVE:
	case E_DRAWING_TOOL:
		break;
	case E_MOUSE_CLICK:
		{
		RTARRAY			output1(fOut1);	
		int				barWidth;
		RCT				gswRect;
		int				dotWidth = 5;

		if (output1.count > 0)							/* a.count can be zero if no data or if the user is not licensed to use PVP */
			{
			int			start, end;

			gswRect.getPaneRect();
			barWidth = getPixelsPerBar();

			mouse.h = pEvent->v.drawtool.where.point.h;
			mouse.v = pEvent->v.drawtool.where.point.v;
			getVisibleBars(&start, &end);
			while (start <= end)
				{
				getDrawPosition(start, output1[start], &pnt);
				pnt.h += barWidth / 2;
				if(mouse.h >= pnt.h - dotWidth && mouse.h <= pnt.h + dotWidth)
					if(mouse.v >= pnt.v - dotWidth && mouse.v <= pnt.v + dotWidth)
						return RTX_OK;
				start++;
				}
			}
		}
		return RTX_FAIL;
	case E_DRAG_DROP:
		break;
	}
return RTX_FAIL;
}
/*****************************/
int cppExtension::calc(int iStartBar)
/*****************************/
{
UserDataPtr pUserData = (UserDataPtr) allocUserData(sizeof(UserData));
/* 6 arrays below are price data pulled from the bars.  The declaration below is all that's needed to load this data. */
RTARRAY		CL(barClose), OP(barOpen), HI(barHigh), LO(barLow);
RTARRAYI	DT(barDateTime), VO(barVolume);
/* The output1 array is used to store the calculations results which persist to the draw function and the next calc call */
RTARRAY		output1(fOut1);
size_t		specSize = (size_t) &pUserData->sRuntime - (size_t) &pUserData->sPeriod;
int			period;
/* iStartBar is passed in and will be 0 on a full calc and CL.count - 3 on a recalc */
int			i = iStartBar;
BOOL		bRecalc = (BOOL)(iStartBar > 0);

/* call below loads parameters into pUserData - don't access pUserData until after this call is made */
/* must send in the size (specSize) of the persistent parameters (minus any runtime parameters) */
if (getParameters(pUserData, specSize, kParmTokens, NULL) == RTX_FAIL)
	return RTX_FAIL;
period = pUserData->sPeriod;
   
if(!bRecalc)
	{
	output1[i] = CL[i];
	i++;
	}

for (; i < CL.count; i++)	/* efficiently compute a simple moving average */
	{
	if(i > period)
		output1[i] = ((output1[i - 1] * (float)(period) - CL[i - period]) + CL[i]) / (float)period;
	else
		output1[i] = (output1[i - 1] * (float)i + CL[i]) / (float)(i + 1);
	}

return RTX_OK;
}
/*****************************/
int myRTExtension::draw(void)
/*****************************/
{
UserDataPtr		pUserData = (UserDataPtr) allocUserData(sizeof(UserData));
RTARRAY			output1(fOut1);
size_t			specSize = (size_t) &pUserData->sRuntime - (size_t) &pUserData->sPeriod;
CPEN			myPen (COLOR_BLACK, 1, P_SOLID);
CBRUSH			myBrush(COLOR_GRAY, PAT_SOLID);
COLOR			paneBackgroundColor = getBackgroundColor();
int				start, end;
BOOL			selected = getSelected();
RCT				gswRect;
int				barWidth = getPixelsPerBar();
int				pixPerPrice = (int)getPixelsPerPrice();
int				i, dotRadius = 3;
RCT				dotRect;
PNT				pnt;

gswRect.getPaneRect();
if (getParameters(pUserData, specSize, kParmTokens, NULL) == RTX_FAIL)
	return RTX_FAIL;

dotRadius = pUserData->dotWidth;
/* start is offset of 1st visible bar and end is offset of last visible bar in chart */
getVisibleBars(&start, &end);
if(selected) /* when indicator is selected, should alter the appearance slightly, often by increasing the pen width by 1 */
	myPen.width++;
/* CPEN myPen is used to control the width, color, and style for drawing lines, and for drawing the border of shapes like rectangles and polygons */
myPen.color = pUserData->colorLine;
myPen.set();
/* CBRUSH myBrush is used for controlinig the color and style when drawing the fill of shapes like rectangels and polygons */
myBrush.color = pUserData->colorDot;
myBrush.set();

for (i = start; i <= end; i++)								// loop through all visibe bars & draw lines and dots
	{
	/* call below gets the bottom left position of price output[i] on bar i*/
	/* pnt.h is the horizontal offset from left of chart while pnt.v is the vertical offset from top of chart */
	getDrawPosition(i, output1[i], &pnt);
	pnt.h += barWidth / 2;
	if(pUserData->bDrawLine)
		{
		if(i > start)
			pnt.drawLineTo();
		}
	if(pUserData->bDrawDot)
		{
		if(i > start)
			dotRect.drawOval();
		dotRect.set(pnt.h - dotRadius, pnt.v - dotRadius, pnt.h + dotRadius, pnt.v + dotRadius);
		if(i == end)
			dotRect.drawOval();
		}
	pnt.setDrawPosition();
	}

/* when performing your own custom drawing as above, you can use the method below to enable "text label" and "label current value" to draw in scale if applicable in indicator preferences */
DRAW_SPECS*		ds;
ds = setArrayDrawingSpecs(1, DRAW_INVISIBLE, myPen.color, myPen.color, 1, DRAW_NO_FLAGS, "Demo");
ds->lineStyle = P_SOLID;
drawArray(output1, ds);

return RTX_OK;
}
/*****************************/
int cppExtension::done(void)
/*****************************/
{
return RTX_OK;
}
/*****************************/
int cppExtension::destroy(void)
/*****************************/
{
freeUserData();
return RTX_OK;
}
/*********** End lsDemo ***********/